@@ -3,9 +3,15 @@
 
 struct CommonCameraInterface
 {
+	virtual ~CommonCameraInterface(){}
 	virtual void getCameraProjectionMatrix(float m[16])const = 0;
 	virtual void getCameraViewMatrix(float m[16]) const = 0;
 		
+	virtual void setVRCamera(const float viewMat[16], const float projectionMatrix[16])=0;
+	virtual void disableVRCamera()=0;
+	virtual bool isVRCamera() const =0;
+	virtual void	setVRCameraOffsetTransform(const float offset[16])=0;
+
 	virtual void getCameraTargetPosition(float pos[3]) const = 0;
 	virtual void getCameraPosition(float pos[3]) const = 0;
 
@@ -30,6 +36,9 @@ struct CommonCameraInterface
 
 	virtual void	setAspectRatio(float ratio) = 0;
 	virtual float	getAspectRatio() const = 0;
+    
+    virtual float getCameraFrustumFar() const = 0;
+    virtual float getCameraFrustumNear() const = 0;
 };
 
 #endif //COMMON_CAMERA_INTERFACE_H
