@@ -13,9 +13,12 @@
 
 class ImportSTLSetup : public CommonRigidBodyBase
 {
+
+	const char* m_fileName;
+	btVector3 m_scaling;
 	
 public:
-    ImportSTLSetup(struct GUIHelperInterface* helper);
+    ImportSTLSetup(struct GUIHelperInterface* helper, const char* fileName);
     virtual ~ImportSTLSetup();
     
 	virtual void initPhysics();
@@ -31,10 +34,19 @@ public:
 };
 
 
-ImportSTLSetup::ImportSTLSetup(struct GUIHelperInterface* helper)
-:CommonRigidBodyBase(helper)
+ImportSTLSetup::ImportSTLSetup(struct GUIHelperInterface* helper, const char* fileName)
+:CommonRigidBodyBase(helper),
+m_scaling(btVector3(10,10,10))
 {
-    
+	if (fileName)
+	{
+		m_fileName = fileName;
+		m_scaling = btVector3(0.01,0.01,0.01);
+	} else
+	{
+		m_fileName = "l_finger_tip.stl";
+
+	}
 }
 
 ImportSTLSetup::~ImportSTLSetup()
@@ -51,17 +63,16 @@ void ImportSTLSetup::initPhysics()
 	m_guiHelper->createPhysicsDebugDrawer(m_dynamicsWorld);
 	m_dynamicsWorld->getDebugDrawer()->setDebugMode(btIDebugDraw::DBG_DrawWireframe);
 
-	const char* fileName = "l_finger_tip.stl";
+	
     char relativeFileName[1024];
-  if (!b3ResourcePath::findResourcePath(fileName, relativeFileName, 1024))
+  if (!b3ResourcePath::findResourcePath(m_fileName, relativeFileName, 1024))
         {
-                b3Warning("Cannot find file %s\n", fileName);
+                b3Warning("Cannot find file %s\n", m_fileName);
                 return;
         }
 
 	
 	btVector3 shift(0,0,0);
-	btVector3 scaling(10,10,10);
 //	int index=10;
 	
 	{
@@ -81,12 +92,12 @@ void ImportSTLSetup::initPhysics()
 		
 		
 	
-		m_guiHelper->getRenderInterface()->registerGraphicsInstance(shapeId,position,orn,color,scaling);
+		m_guiHelper->getRenderInterface()->registerGraphicsInstance(shapeId,position,orn,color,m_scaling);
 
 	}
 }
 
 class CommonExampleInterface*    ImportSTLCreateFunc(struct CommonExampleOptions& options)
 {
-	return new ImportSTLSetup(options.m_guiHelper);
+	return new ImportSTLSetup(options.m_guiHelper, options.m_fileName);
 }
