@@ -135,8 +135,8 @@ void TestJointTorqueSetup::initPhysics()
             delete shape;
         }
 
-        bool isMultiDof = true;
-        btMultiBody *pMultiBody = new btMultiBody(numLinks, baseMass, baseInertiaDiag, !floating, canSleep, isMultiDof);
+
+        btMultiBody *pMultiBody = new btMultiBody(numLinks, baseMass, baseInertiaDiag, !floating, canSleep);
 		
         m_multiBody = pMultiBody;
         btQuaternion baseOriQuat(0.f, 0.f, 0.f, 1.f);
@@ -200,7 +200,7 @@ void TestJointTorqueSetup::initPhysics()
 					pMultiBody->setupFixed(i, linkMass, linkInertiaDiag, i - 1, 
 					btQuaternion(0.f, 0.f, 0.f, 1.f), 
 					parentComToCurrentPivot, 
-					currentPivotToCurrentCom, false);
+					currentPivotToCurrentCom);
 				}
 					
 				//pMultiBody->setupFixed(i,linkMass,linkInertiaDiag,i-1,btQuaternion(0,0,0,1),parentComToCurrentPivot,currentPivotToCurrentCom,false);
@@ -248,10 +248,9 @@ void TestJointTorqueSetup::initPhysics()
         {
             btScalar q0 = 45.f * SIMD_PI/ 180.f;
             if(!spherical)
-                if(mbC->isMultiDof())
-                    mbC->setJointPosMultiDof(0, &q0);
-                else
-                    mbC->setJointPos(0, q0);
+			{
+				mbC->setJointPosMultiDof(0, &q0);
+			}
             else
             {
                 btQuaternion quat0(btVector3(1, 1, 0).normalized(), q0);
@@ -295,8 +294,8 @@ void TestJointTorqueSetup::initPhysics()
                 col->setWorldTransform(tr);
 
 				bool isDynamic = (baseMass > 0 && floating);
-				short collisionFilterGroup = isDynamic? short(btBroadphaseProxy::DefaultFilter) : short(btBroadphaseProxy::StaticFilter);
-				short collisionFilterMask = isDynamic? 	short(btBroadphaseProxy::AllFilter) : 	short(btBroadphaseProxy::AllFilter ^ btBroadphaseProxy::StaticFilter);
+				int collisionFilterGroup = isDynamic? int(btBroadphaseProxy::DefaultFilter) : int(btBroadphaseProxy::StaticFilter);
+				int collisionFilterMask = isDynamic? 	int(btBroadphaseProxy::AllFilter) : 	int(btBroadphaseProxy::AllFilter ^ btBroadphaseProxy::StaticFilter);
 
 
                 world->addCollisionObject(col,collisionFilterGroup,collisionFilterMask);//, 2,1+2);
@@ -348,8 +347,8 @@ void TestJointTorqueSetup::initPhysics()
             col->setWorldTransform(tr);
      //       col->setFriction(friction);
 			bool isDynamic = 1;//(linkMass > 0);
-			short collisionFilterGroup = isDynamic? short(btBroadphaseProxy::DefaultFilter) : short(btBroadphaseProxy::StaticFilter);
-			short collisionFilterMask = isDynamic? 	short(btBroadphaseProxy::AllFilter) : 	short(btBroadphaseProxy::AllFilter ^ btBroadphaseProxy::StaticFilter);
+			int collisionFilterGroup = isDynamic? int(btBroadphaseProxy::DefaultFilter) : int(btBroadphaseProxy::StaticFilter);
+			int collisionFilterMask = isDynamic? 	int(btBroadphaseProxy::AllFilter) : 	int(btBroadphaseProxy::AllFilter ^ btBroadphaseProxy::StaticFilter);
 
 			//if (i==0||i>numLinks-2)
 			{
@@ -368,9 +367,8 @@ void TestJointTorqueSetup::initPhysics()
 
 	btSerializer* s = new btDefaultSerializer;
 	m_dynamicsWorld->serialize(s);
-	b3ResourcePath p;
 	char resourcePath[1024];
-	if (p.findResourcePath("multibody.bullet",resourcePath,1024))
+	if (b3ResourcePath::findResourcePath("multibody.bullet",resourcePath,1024))
 	{
 		FILE* f = fopen(resourcePath,"wb");
 		fwrite(s->getBufferPointer(),s->getCurrentBufferSize(),1,f);
