@@ -10,6 +10,61 @@
 #include <stddef.h>
 #include <string.h>
 
+#include <Availability.h>
+
+
+//aargh, Mac OSX 10.12 broke backwards compatibility, fix it here for now
+#ifdef __MAC_10_12
+    #define MyNSTitledWindowMask NSWindowStyleMaskTitled
+    #define MyNSResizableWindowMask NSWindowStyleMaskResizable
+    #define MyNSClosableWindowMask NSWindowStyleMaskClosable
+    #define MyNSMiniaturizableWindowMask NSWindowStyleMaskMiniaturizable
+    #define MyNSAnyEventMask NSEventMaskAny
+    #define MyNSEventTypeFlagsChanged NSEventTypeFlagsChanged
+    #define MyNSEventModifierFlagShift NSEventModifierFlagShift
+    #define MyNSEventModifierFlagControl NSEventModifierFlagControl
+    #define MyNSEventModifierFlagOption NSEventModifierFlagOption
+    #define MyNSKeyUp NSEventTypeKeyUp
+    #define MyNSKeyDown NSEventTypeKeyDown
+    #define MyNSRightMouseDown NSEventTypeRightMouseDown
+    #define MyNSLeftMouseDown NSEventTypeLeftMouseDown
+    #define MyNSOtherMouseDown NSEventTypeOtherMouseDown
+    #define MyNSRightMouseUp NSEventTypeRightMouseUp
+    #define MyNSLeftMouseUp NSEventTypeLeftMouseUp
+    #define MyNSOtherMouseUp NSEventTypeOtherMouseUp
+    #define MyNSMouseMoved NSEventTypeMouseMoved
+    #define MyNSLeftMouseDragged NSEventTypeLeftMouseDragged
+    #define MyNSRightMouseDragged NSEventTypeRightMouseDragged
+    #define MyNSOtherMouseDragged NSEventTypeOtherMouseDragged
+    #define MyNSScrollWheel NSEventTypeScrollWheel
+
+
+#else
+    #define MyNSTitledWindowMask NSTitledWindowMask
+    #define MyNSResizableWindowMask NSResizableWindowMask
+    #define MyNSClosableWindowMask NSClosableWindowMask
+    #define MyNSMiniaturizableWindowMask NSMiniaturizableWindowMask
+    #define MyNSAnyEventMask NSAnyEventMask
+    #define MyNSEventTypeFlagsChanged NSFlagsChanged
+    #define MyNSEventModifierFlagShift NSShiftKeyMask
+    #define MyNSEventModifierFlagControl NSControlKeyMask
+    #define MyNSEventModifierFlagOption NSAlternateKeyMask
+    #define MyNSKeyUp NSKeyUp
+    #define MyNSKeyDown NSKeyDown
+    #define MyNSRightMouseDown NSRightMouseDown
+    #define MyNSLeftMouseDown NSLeftMouseDown
+    #define MyNSOtherMouseDown NSOtherMouseDown
+    #define MyNSLeftMouseUp NSLeftMouseUp
+    #define MyNSRightMouseUp NSRightMouseUp
+    #define MyNSOtherMouseUp NSOtherMouseUp
+    #define MyNSMouseMoved NSMouseMoved
+    #define MyNSLeftMouseDragged NSLeftMouseDragged
+    #define MyNSRightMouseDragged NSRightMouseDragged
+    #define MyNSOtherMouseDragged NSOtherMouseDragged
+    #define MyNSScrollWheel NSScrollWheel
+
+
+#endif
 
 enum
 {
@@ -102,11 +157,11 @@ float loop;
 }
 -(void)drawRect:(NSRect)rect
 {
+	
 	if (([self frame].size.width != m_lastWidth) || ([self frame].size.height != m_lastHeight))
 	{
 		m_lastWidth = [self frame].size.width;
 		m_lastHeight = [self frame].size.height;
-		
 		// Only needed on resize:
 		[m_context clearDrawable];
 		
@@ -114,7 +169,6 @@ float loop;
         float width = [self frame].size.width;
         float height = [self frame].size.height;
         
-        
         // Get view dimensions in pixels
      //   glViewport(0,0,10,10);
         
@@ -209,16 +263,12 @@ struct MacOpenGLWindowInternalData
         m_myview = 0;
         m_pool = 0;
         m_window = 0;
-        m_width = -1;
-        m_height = -1;
         m_exitRequested = false;
     }
     NSApplication*      m_myApp;
     TestView*             m_myview;
     NSAutoreleasePool*  m_pool;
     NSWindow*           m_window;
-    int m_width;
-    int m_height;
     bool m_exitRequested;
     
 };
@@ -228,8 +278,8 @@ MacOpenGLWindow::MacOpenGLWindow()
 m_mouseX(0),
 m_mouseY(0),
 m_modifierFlags(0),
-m_mouseMoveCallback(0),
 m_mouseButtonCallback(0),
+m_mouseMoveCallback(0),
 m_wheelCallback(0),
 m_keyboardCallback(0),
 m_retinaScaleFactor(1),
@@ -294,8 +344,6 @@ void MacOpenGLWindow::createWindow(const b3gWindowConstructionInfo& ci)
     if (m_internalData)
         closeWindow();
 
-    int width = ci.m_width;
-	int height = ci.m_height;
 	const char* windowTitle = ci.m_title;
 	
 
@@ -303,9 +351,7 @@ void MacOpenGLWindow::createWindow(const b3gWindowConstructionInfo& ci)
     NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
     
     m_internalData = new MacOpenGLWindowInternalData;
-    m_internalData->m_width = width;
-    m_internalData->m_height = height;
-    
+  
     m_internalData->m_pool = [NSAutoreleasePool new];
 	m_internalData->m_myApp = [NSApplication sharedApplication];
 	//myApp = [MyApp sharedApplication];
@@ -343,10 +389,10 @@ void MacOpenGLWindow::createWindow(const b3gWindowConstructionInfo& ci)
     
     // add Edit menu
     NSMenuItem *editMenuItem = [[NSMenuItem new] autorelease];
-    NSMenu *menu = [[NSMenu allocWithZone:[NSMenu menuZone]]initWithTitle:@"Edit"];
+    NSMenu *menu = [[NSMenu alloc]initWithTitle:@"Edit"];
     [editMenuItem setSubmenu: menu];
     
-    NSMenuItem *copyItem = [[NSMenuItem allocWithZone:[NSMenu menuZone]]initWithTitle:@"Copy" action:@selector(copy:) keyEquivalent:@"c"];
+    NSMenuItem *copyItem = [[NSMenuItem alloc]initWithTitle:@"Copy" action:@selector(copy:) keyEquivalent:@"c"];
     
     [menu addItem:copyItem];
     [menubar addItem:editMenuItem];
@@ -373,11 +419,11 @@ void MacOpenGLWindow::createWindow(const b3gWindowConstructionInfo& ci)
     [newItem release];
     */
     
-	NSRect frame = NSMakeRect(0., 0., width, height);
+	NSRect frame = NSMakeRect(0., 0., ci.m_width, ci.m_height);
 	
 	m_internalData->m_window = [NSWindow alloc];
 	[m_internalData->m_window initWithContentRect:frame
-                      styleMask:NSTitledWindowMask |NSResizableWindowMask| NSClosableWindowMask | NSMiniaturizableWindowMask
+                      styleMask:MyNSTitledWindowMask |MyNSResizableWindowMask| MyNSClosableWindowMask | MyNSMiniaturizableWindowMask
                         backing:NSBackingStoreBuffered
                           defer:false];
 	
@@ -423,9 +469,7 @@ void MacOpenGLWindow::createWindow(const b3gWindowConstructionInfo& ci)
     [m_internalData->m_window makeKeyAndOrderFront: nil];
     
     [m_internalData->m_myview MakeCurrent];
-    //m_internalData->m_width = m_internalData->m_myview.GetWindowWidth;
-    //m_internalData->m_height = m_internalData->m_myview.GetWindowHeight;
-    
+	
     
     [NSApp activateIgnoringOtherApps:YES];
     
@@ -782,7 +826,7 @@ void MacOpenGLWindow::startRendering()
         [pool release];
         pool = [[NSAutoreleasePool alloc] init];
         event =        [m_internalData->m_myApp
-         nextEventMatchingMask:NSAnyEventMask
+         nextEventMatchingMask:MyNSAnyEventMask
          untilDate:[NSDate distantPast]
          inMode:NSDefaultRunLoopMode
          //		  inMode:NSEventTrackingRunLoopMode
@@ -791,12 +835,12 @@ void MacOpenGLWindow::startRendering()
 		//NSShiftKeyMask              = 1 << 17,
 		//NSControlKeyMask
 	
-		if ([event type] == NSFlagsChanged)
+		if ([event type] == MyNSEventTypeFlagsChanged)
 		{
 			int modifiers = [event modifierFlags];
 			if (m_keyboardCallback)
 			{
-				if ((modifiers & NSShiftKeyMask))
+				if ((modifiers & MyNSEventModifierFlagShift))
 				{
 					m_keyboardCallback(B3G_SHIFT,1);
 					m_modifierFlags |= MY_MAC_SHIFTKEY;
@@ -808,7 +852,7 @@ void MacOpenGLWindow::startRendering()
 						m_modifierFlags &= ~MY_MAC_SHIFTKEY;
 					}
 				}
-				if (modifiers & NSControlKeyMask)
+				if (modifiers & MyNSEventModifierFlagControl)
 				{
 					m_keyboardCallback(B3G_CONTROL,1);
 					m_modifierFlags |= MY_MAC_CONTROL_KEY;
@@ -820,7 +864,7 @@ void MacOpenGLWindow::startRendering()
 						m_modifierFlags &= ~MY_MAC_CONTROL_KEY;
 					}
 				}
-				if (modifiers & NSAlternateKeyMask)
+				if (modifiers & MyNSEventModifierFlagOption)
 				{
 					m_keyboardCallback(B3G_ALT,1);
 					m_modifierFlags |= MY_MAC_ALTKEY;
@@ -837,7 +881,7 @@ void MacOpenGLWindow::startRendering()
 				
 			}
 		}
-		if ([event type] == NSKeyUp)
+		if ([event type] == MyNSKeyUp)
         {
 			handledEvent = true;
 			
@@ -852,7 +896,9 @@ void MacOpenGLWindow::startRendering()
 				m_keyboardCallback(keycode,state);
 			}
 		}
-        if ([event type] == NSKeyDown)
+
+        
+        if ([event type] == MyNSKeyDown)
         {
 			handledEvent = true;
 			
@@ -872,7 +918,8 @@ void MacOpenGLWindow::startRendering()
 		}
 
 
-        if (([event type]== NSRightMouseDown) || ([ event type]==NSLeftMouseDown)||([event type]==NSOtherMouseDown))
+        
+        if (([event type]== MyNSRightMouseDown) || ([ event type]==MyNSLeftMouseDown)||([event type]==MyNSOtherMouseDown))
         {
            // printf("right mouse!");
           //  float mouseX,mouseY;
@@ -884,17 +931,17 @@ void MacOpenGLWindow::startRendering()
             int button=0;
 	        switch ([event type])
             {
-                case NSLeftMouseDown:
+                case MyNSLeftMouseDown:
                 {
                     button=0;
                     break;
                 }
-                case NSOtherMouseDown:
+                case MyNSOtherMouseDown:
                 {
                     button=1;
                     break;
                 }
-                case NSRightMouseDown:
+                case MyNSRightMouseDown:
                 {
                     button=2;
                     break;
@@ -913,7 +960,7 @@ void MacOpenGLWindow::startRendering()
         }
 		
 		
-        if (([event type]== NSRightMouseUp) || ([ event type]==NSLeftMouseUp)||([event type]==NSOtherMouseUp))
+        if (([event type]== MyNSRightMouseUp) || ([ event type]==MyNSLeftMouseUp)||([event type]==MyNSOtherMouseUp))
         {
 			// printf("right mouse!");
 			//  float mouseX,mouseY;
@@ -926,17 +973,17 @@ void MacOpenGLWindow::startRendering()
             int button=0;
             switch ([event type])
             {
-                case NSLeftMouseUp:
+                case MyNSLeftMouseUp:
                 {
                     button=0;
                     break;
                 }
-                case NSOtherMouseUp:
+                case MyNSOtherMouseUp:
                 {
                     button=1;
                     break;
                 }
-                case NSRightMouseUp:
+                case MyNSRightMouseUp:
                 {
                     button=2;
                     break;
@@ -954,7 +1001,7 @@ void MacOpenGLWindow::startRendering()
         }
 		
         
-        if ([event type] == NSMouseMoved)
+        if ([event type] == MyNSMouseMoved)
         {
             
             NSPoint eventLocation = [event locationInWindow];
@@ -971,7 +1018,8 @@ void MacOpenGLWindow::startRendering()
 			}
         }
         
-        if (([event type] == NSLeftMouseDragged) || ([event type] == NSRightMouseDragged) || ([event type] == NSOtherMouseDragged))
+        
+        if (([event type] == MyNSLeftMouseDragged) || ([event type] == MyNSRightMouseDragged) || ([event type] == MyNSOtherMouseDragged))
         {
             int dx1, dy1;
             CGGetLastMouseDelta (&dx1, &dy1);
@@ -990,7 +1038,7 @@ void MacOpenGLWindow::startRendering()
           //  printf("mouse coord = %f, %f\n",m_mouseX,m_mouseY);
         }
         
-        if ([event type] == NSScrollWheel)
+        if ([event type] == MyNSScrollWheel)
         {
             float dy, dx;
             dy = [ event deltaY ];
@@ -1035,13 +1083,13 @@ void MacOpenGLWindow::startRendering()
     float aspect;
     //b3Vector3 extents;
     
-    if (m_internalData->m_width > m_internalData->m_height)
+    if (getWidth()  > getHeight())
     {
-        aspect = (float)m_internalData->m_width / (float)m_internalData->m_height;
+        aspect = (float)getWidth() / (float)getHeight();
         //extents.setValue(aspect * 1.0f, 1.0f,0);
     } else
     {
-        aspect = (float)m_internalData->m_height / (float)m_internalData->m_width;
+        aspect = (float)getHeight() / (float)getWidth();
         //extents.setValue(1.0f, aspect*1.f,0);
     }
     
@@ -1082,7 +1130,7 @@ int MacOpenGLWindow::fileOpenDialog(char* filename, int maxNameLength)
     NSOpenGLContext *foo = [NSOpenGLContext currentContext];
     // get the url of a .txt file
     NSOpenPanel * zOpenPanel = [NSOpenPanel openPanel];
-	NSArray * zAryOfExtensions = [NSArray arrayWithObjects:@"urdf",@"bullet",nil];
+	NSArray * zAryOfExtensions = [NSArray arrayWithObjects:@"urdf",@"bullet",@"obj",@"sdf",@"stl",nil];
     [zOpenPanel setAllowedFileTypes:zAryOfExtensions];
     NSInteger zIntResult = [zOpenPanel runModal];
     
@@ -1132,12 +1180,28 @@ void MacOpenGLWindow::getMouseCoordinates(int& x, int& y)
     
 }
 
+int   MacOpenGLWindow::getWidth() const
+{
+    if (m_internalData && m_internalData->m_myview && m_internalData->m_myview.GetWindowWidth)
+        return m_internalData->m_myview.GetWindowWidth;
+
+    return 0;
+}
+
+int   MacOpenGLWindow::getHeight() const
+{
+    if (m_internalData && m_internalData->m_myview && m_internalData->m_myview.GetWindowHeight)
+        return m_internalData->m_myview.GetWindowHeight;
+    return 0;
+}
+
+
 void MacOpenGLWindow::setResizeCallback(b3ResizeCallback resizeCallback)
 {
     [m_internalData->m_myview setResizeCallback:resizeCallback];
     if (resizeCallback)
     {
-        (resizeCallback)(m_internalData->m_width,m_internalData->m_height);
+		(resizeCallback)(getWidth(), getHeight());
     }
 }
 
