@@ -24,14 +24,16 @@ struct SimpleOpenGL3App : public CommonGraphicsApp
 	virtual void registerGrid(int xres, int yres, float color0[4], float color1[4]);
     void dumpNextFrameToPng(const char* pngFilename);
     void dumpFramesToVideo(const char* mp4Filename);
+    void getScreenPixels(unsigned char* rgbaBuffer, int bufferSizeInBytes, float* depthBuffer, int depthBufferSizeInBytes);
     
 	void drawGrid(DrawGridData data=DrawGridData());
 	virtual void setUpAxis(int axis);
 	virtual int getUpAxis() const;
 	
 	virtual void swapBuffer();
-	virtual void drawText( const char* txt, int posX, int posY);
+	virtual void drawText( const char* txt, int posX, int posY, float size=1.0f);
 	virtual void drawText3D( const char* txt, float posX, float posZY, float posZ, float size);
+	virtual void drawTexturedRect(float x0, float y0, float x1, float y1, float color[4], float u0,float v0, float u1, float v1, int useRGBA);
 	struct sth_stash* getFontStash();
 
 
