@@ -172,6 +172,8 @@ struct InternalData2
     XEvent                  m_xev;
     GLXFBConfig             m_bestFbc;
     int			    m_modifierFlags;
+    int			    m_glWidth;
+    int			    m_glHeight;
 
 #ifdef DYNAMIC_LOAD_X11_FUNCTIONS
 	//dynamically load stuff
@@ -207,6 +209,9 @@ struct InternalData2
 	InternalData2()
 	:m_dpy(0),
 	m_vi(0),
+	m_modifierFlags(0),
+	m_glWidth(-1),
+	m_glHeight(-1),
 	m_wheelCallback(0),
 	m_mouseMoveCallback(0),
 	m_mouseButtonCallback(0),
@@ -456,6 +461,7 @@ void X11OpenGLWindow::enableOpenGL()
 
   printf( "Making context current\n" );
   glXMakeCurrent( m_data->m_dpy, m_data->m_win, ctx );
+  m_data->m_glc = ctx;
 
     } else
     {
@@ -510,6 +516,9 @@ void    X11OpenGLWindow::createWindow(const b3gWindowConstructionInfo& ci)
 
     m_data->m_dpy = MyXOpenDisplay(NULL);
 
+    m_data->m_glWidth = ci.m_width;
+    m_data->m_glHeight = ci.m_height;
+
     if(m_data->m_dpy == NULL) {
         printf("\n\tcannot connect to X server\n\n");
             exit(0);
@@ -830,7 +839,7 @@ void X11OpenGLWindow::pumpMessage()
 
                 if (m_data->m_keyboardCallback)
                 {
-#if 0
+#if 1
                      unsigned short is_retriggered = 0;
 ///filter out keyboard repeat
 //see http://stackoverflow.com/questions/2100654/ignore-auto-repeat-in-x11-applications
@@ -842,8 +851,8 @@ void X11OpenGLWindow::pumpMessage()
                          if (nev.type == KeyPress && nev.xkey.time ==  m_data->m_xev.xkey.time &&
                              nev.xkey.keycode ==  m_data->m_xev.xkey.keycode)
                            {
-                             fprintf (stdout, "key #%ld was retriggered.\n",
-                               (long) MyXLookupKeysym(&nev.xkey, 0));
+                             //fprintf (stdout, "key #%ld was retriggered.\n",
+                              // (long) MyXLookupKeysym(&nev.xkey, 0));
 
                              // delete retriggered KeyPress event
                              MyXNextEvent(m_data->m_dpy, & m_data->m_xev);
@@ -852,6 +861,7 @@ void X11OpenGLWindow::pumpMessage()
                        }
 #endif
                     int state = 0;
+		    if (!is_retriggered)
                     (*m_data->m_keyboardCallback)(keycode,state);
                     }
 
@@ -929,6 +939,9 @@ void X11OpenGLWindow::pumpMessage()
             {
   //              printf("@");
   //              fflush(0);
+		m_data->m_glWidth = m_data->m_xev.xconfigure.width;
+		m_data->m_glHeight = m_data->m_xev.xconfigure.height;
+
                 if (m_data->m_resizeCallback)
                 {
                     (*m_data->m_resizeCallback)(m_data->m_xev.xconfigure.width,m_data->m_xev.xconfigure.height);
@@ -1031,6 +1044,10 @@ void X11OpenGLWindow::setMouseButtonCallback(b3MouseButtonCallback	mouseCallback
 
 void X11OpenGLWindow::setResizeCallback(b3ResizeCallback	resizeCallback)
 {
+	if (resizeCallback && m_data->m_glWidth>0 && m_data->m_glHeight > 0)
+	{
+		resizeCallback(m_data->m_glWidth, m_data->m_glHeight);
+	}
 	m_data->m_resizeCallback = resizeCallback;
 }
 
@@ -1063,12 +1080,26 @@ b3KeyboardCallback      X11OpenGLWindow::getKeyboardCallback()
 	return m_data->m_keyboardCallback;
 }
 
+int   X11OpenGLWindow::getWidth() const
+{
+    if (m_data)
+        return m_data->m_glWidth;
+    return 0;
+}
+int   X11OpenGLWindow::getHeight() const
+{
+    if (m_data)
+        return m_data->m_glHeight;
+    return 0;
+}
+
+
 #include <stdio.h>
 
 int X11OpenGLWindow::fileOpenDialog(char* filename, int maxNameLength)
 {
 	int len = 0;
-	FILE * output = popen("zenity --file-selection --file-filter=\"*.urdf\" --file-filter=\"*.*\"","r");
+	FILE * output = popen("zenity --file-selection --file-filter=\"*.urdf\" --file-filter=\"*.sdf\"  --file-filter=\"*.obj\"  --file-filter=\"*.*\"","r");
 	if (output)
 	{
 		while( fgets(filename, maxNameLength-1, output) != NULL )
@@ -1078,7 +1109,7 @@ int X11OpenGLWindow::fileOpenDialog(char* filename, int maxNameLength)
 			{
 				filename[len-1]=0;
 				printf("file open (length=%d) = %s\n", len,filename);
-			}	
+			}
 		}
 		pclose(output);
 	} else
@@ -1087,5 +1118,5 @@ int X11OpenGLWindow::fileOpenDialog(char* filename, int maxNameLength)
 	}
 	MyXRaiseWindow(m_data->m_dpy, m_data->m_win);
 	return len;
-	
+
 }
