@@ -15,10 +15,8 @@
 
 #include "btQuickprof.h"
 
-#ifndef BT_NO_PROFILE
 
 
-static btClock gProfileClock;
 
 
 #ifdef __CELLOS_LV2__
@@ -30,6 +28,10 @@ static btClock gProfileClock;
 #if defined (SUNOS) || defined (__SUNOS__)
 #include <stdio.h>
 #endif
+#ifdef __APPLE__
+#include <mach/mach_time.h>
+#include <TargetConditionals.h>
+#endif
 
 #if defined(WIN32) || defined(_WIN32)
 
@@ -44,8 +46,8 @@ static btClock gProfileClock;
 #else //_XBOX
 	#include <windows.h>
 
-#if WINVER < 0x0600
-ULONGLONG GetTickCount64() { return GetTickCount(); }
+#if WINVER <0x0602
+#define GetTickCount64 GetTickCount
 #endif
 
 #endif //_XBOX
@@ -55,6 +57,12 @@ ULONGLONG GetTickCount64() { return GetTickCount(); }
 
 #else //_WIN32
 #include <sys/time.h>
+
+#ifdef BT_LINUX_REALTIME
+//required linking against rt (librt)
+#include <time.h>
+#endif //BT_LINUX_REALTIME
+
 #endif //_WIN32
 
 #define mymin(a,b) (a > b ? a : b)
@@ -65,12 +73,14 @@ struct btClockData
 #ifdef BT_USE_WINDOWS_TIMERS
 	LARGE_INTEGER mClockFrequency;
 	LONGLONG mStartTick;
-	LONGLONG mPrevElapsedTime;
 	LARGE_INTEGER mStartTime;
 #else
 #ifdef __CELLOS_LV2__
 	uint64_t	mStartTime;
 #else
+#ifdef __APPLE__
+    uint64_t mStartTimeNano;
+#endif
 	struct timeval mStartTime;
 #endif
 #endif //__CELLOS_LV2__
@@ -111,7 +121,6 @@ void btClock::reset()
 #ifdef BT_USE_WINDOWS_TIMERS
 	QueryPerformanceCounter(&m_data->mStartTime);
 	m_data->mStartTick = GetTickCount64();
-	m_data->mPrevElapsedTime = 0;
 #else
 #ifdef __CELLOS_LV2__
 
@@ -121,6 +130,9 @@ void btClock::reset()
 	SYS_TIMEBASE_GET( newTime );
 	m_data->mStartTime = newTime;
 #else
+#ifdef __APPLE__
+    m_data->mStartTimeNano = mach_absolute_time();
+#endif
 	gettimeofday(&m_data->mStartTime, 0);
 #endif
 #endif
@@ -128,7 +140,7 @@ void btClock::reset()
 
 /// Returns the time in ms since the last call to reset or since
 /// the btClock was created.
-unsigned long int btClock::getTimeMilliseconds()
+unsigned long long int btClock::getTimeMilliseconds()
 {
 #ifdef BT_USE_WINDOWS_TIMERS
 	LARGE_INTEGER currentTime;
@@ -138,27 +150,6 @@ unsigned long int btClock::getTimeMilliseconds()
 		// Compute the number of millisecond ticks elapsed.
 	unsigned long msecTicks = (unsigned long)(1000 * elapsedTime /
 		m_data->mClockFrequency.QuadPart);
-		// Check for unexpected leaps in the Win32 performance counter.
-		// (This is caused by unexpected data across the PCI to ISA
-		// bridge, aka south bridge.  See Microsoft KB274323.)
-		unsigned long elapsedTicks = (unsigned long)(GetTickCount64() - m_data->mStartTick);
-		signed long msecOff = (signed long)(msecTicks - elapsedTicks);
-		if (msecOff < -100 || msecOff > 100)
-		{
-			// Adjust the starting time forwards.
-			LONGLONG msecAdjustment = mymin(msecOff *
-				m_data->mClockFrequency.QuadPart / 1000, elapsedTime -
-				m_data->mPrevElapsedTime);
-			m_data->mStartTime.QuadPart += msecAdjustment;
-			elapsedTime -= msecAdjustment;
-
-			// Recompute the number of millisecond ticks elapsed.
-			msecTicks = (unsigned long)(1000 * elapsedTime /
-				m_data->mClockFrequency.QuadPart);
-		}
-
-		// Store the current elapsed time for adjustments next time.
-		m_data->mPrevElapsedTime = elapsedTime;
 
 		return msecTicks;
 #else
@@ -184,41 +175,19 @@ unsigned long int btClock::getTimeMilliseconds()
 
 	/// Returns the time in us since the last call to reset or since
 	/// the Clock was created.
-unsigned long int btClock::getTimeMicroseconds()
+unsigned long long int btClock::getTimeMicroseconds()
 {
 #ifdef BT_USE_WINDOWS_TIMERS
-		LARGE_INTEGER currentTime;
+	//see https://msdn.microsoft.com/en-us/library/windows/desktop/dn553408(v=vs.85).aspx	
+		LARGE_INTEGER currentTime, elapsedTime;
+		
 		QueryPerformanceCounter(&currentTime);
-		LONGLONG elapsedTime = currentTime.QuadPart -
+		elapsedTime.QuadPart = currentTime.QuadPart - 
 			m_data->mStartTime.QuadPart;
+		elapsedTime.QuadPart *= 1000000;
+		elapsedTime.QuadPart /= m_data->mClockFrequency.QuadPart;
 
-		// Compute the number of millisecond ticks elapsed.
-		unsigned long msecTicks = (unsigned long)(1000 * elapsedTime /
-			m_data->mClockFrequency.QuadPart);
-
-		// Check for unexpected leaps in the Win32 performance counter.
-		// (This is caused by unexpected data across the PCI to ISA
-		// bridge, aka south bridge.  See Microsoft KB274323.)
-		unsigned long elapsedTicks = (unsigned long)(GetTickCount64() - m_data->mStartTick);
-		signed long msecOff = (signed long)(msecTicks - elapsedTicks);
-		if (msecOff < -100 || msecOff > 100)
-		{
-			// Adjust the starting time forwards.
-			LONGLONG msecAdjustment = mymin(msecOff *
-				m_data->mClockFrequency.QuadPart / 1000, elapsedTime -
-				m_data->mPrevElapsedTime);
-			m_data->mStartTime.QuadPart += msecAdjustment;
-			elapsedTime -= msecAdjustment;
-		}
-
-		// Store the current elapsed time for adjustments next time.
-		m_data->mPrevElapsedTime = elapsedTime;
-
-		// Convert to microseconds.
-		unsigned long usecTicks = (unsigned long)(1000000 * elapsedTime /
-			m_data->mClockFrequency.QuadPart);
-
-		return usecTicks;
+		return (unsigned long long) elapsedTime.QuadPart;
 #else
 
 #ifdef __CELLOS_LV2__
@@ -240,6 +209,66 @@ unsigned long int btClock::getTimeMicroseconds()
 #endif
 }
 
+unsigned long long int btClock::getTimeNanoseconds()
+{
+#ifdef BT_USE_WINDOWS_TIMERS
+	//see https://msdn.microsoft.com/en-us/library/windows/desktop/dn553408(v=vs.85).aspx
+		LARGE_INTEGER currentTime, elapsedTime;
+		
+		QueryPerformanceCounter(&currentTime);
+		elapsedTime.QuadPart = currentTime.QuadPart - 
+			m_data->mStartTime.QuadPart;
+		elapsedTime.QuadPart *= 1e9;
+		elapsedTime.QuadPart /= m_data->mClockFrequency.QuadPart;
+
+		return (unsigned long long) elapsedTime.QuadPart;
+#else
+
+#ifdef __CELLOS_LV2__
+		uint64_t freq=sys_time_get_timebase_frequency();
+		double dFreq=((double) freq)/ 1e9;
+		typedef uint64_t  ClockSize;
+		ClockSize newTime;
+		//__asm __volatile__( "mftb %0" : "=r" (newTime) : : "memory");
+		SYS_TIMEBASE_GET( newTime );
+
+		return (unsigned long int)((double(newTime-m_data->mStartTime)) / dFreq);
+#else
+#ifdef __APPLE__
+    uint64_t ticks = mach_absolute_time() - m_data->mStartTimeNano;
+    static long double conversion = 0.0L;
+    if( 0.0L == conversion )
+    {
+        // attempt to get conversion to nanoseconds
+        mach_timebase_info_data_t info;
+        int err = mach_timebase_info( &info );
+        if( err )
+        {
+            btAssert(0);
+            conversion = 1.;
+        }
+        conversion = info.numer / info.denom;
+    }
+    return (ticks * conversion);
+
+    
+#else//__APPLE__
+    
+#ifdef BT_LINUX_REALTIME
+    timespec ts;
+    clock_gettime(CLOCK_REALTIME,&ts);
+    return 1000000000*ts.tv_sec + ts.tv_nsec;
+#else
+    	struct timeval currentTime;
+		gettimeofday(&currentTime, 0);
+		return (currentTime.tv_sec - m_data->mStartTime.tv_sec) * 1e9 +
+			(currentTime.tv_usec - m_data->mStartTime.tv_usec)*1000;
+#endif //BT_LINUX_REALTIME
+
+#endif//__APPLE__
+#endif//__CELLOS_LV2__
+#endif 
+}
 
 
 /// Returns the time in s since the last call to reset or since 
@@ -250,6 +279,10 @@ btScalar btClock::getTimeSeconds()
 	return btScalar(getTimeMicroseconds()) * microseconds_to_seconds;
 }
 
+#ifndef BT_NO_PROFILE
+
+
+static btClock gProfileClock;
 
 
 inline void Profile_Get_Ticks(unsigned long int * ticks)
@@ -265,7 +298,6 @@ inline float Profile_Get_Tick_Rate(void)
 }
 
 
-
 /***************************************************************************************************
 **
 ** CProfileNode
@@ -367,6 +399,7 @@ bool	CProfileNode::Return( void )
 	if ( --RecursionCounter == 0 && TotalCalls != 0 ) {
 		unsigned long int time;
 		Profile_Get_Ticks(&time);
+
 		time-=StartTime;
 		TotalTime += (float)time / Profile_Get_Tick_Rate();
 	}
@@ -434,11 +467,71 @@ void	CProfileIterator::Enter_Parent( void )
 **
 ***************************************************************************************************/
 
-CProfileNode	CProfileManager::Root( "Root", NULL );
-CProfileNode *	CProfileManager::CurrentNode = &CProfileManager::Root;
+
+
+
+CProfileNode	gRoots[BT_QUICKPROF_MAX_THREAD_COUNT]={
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),
+	CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL),CProfileNode("Root",NULL)
+};
+
+
+CProfileNode* gCurrentNodes[BT_QUICKPROF_MAX_THREAD_COUNT]=
+{
+	&gRoots[ 0],	&gRoots[ 1],	&gRoots[ 2],	&gRoots[ 3],
+	&gRoots[ 4],	&gRoots[ 5],	&gRoots[ 6],	&gRoots[ 7],
+	&gRoots[ 8],	&gRoots[ 9],	&gRoots[10],	&gRoots[11],
+	&gRoots[12],	&gRoots[13],	&gRoots[14],	&gRoots[15],
+	&gRoots[16],	&gRoots[17],	&gRoots[18],	&gRoots[19],
+	&gRoots[20],	&gRoots[21],	&gRoots[22],	&gRoots[23],
+	&gRoots[24],	&gRoots[25],	&gRoots[26],	&gRoots[27],
+	&gRoots[28],	&gRoots[29],	&gRoots[30],	&gRoots[31],
+	&gRoots[32],	&gRoots[33],	&gRoots[34],	&gRoots[35],
+	&gRoots[36],	&gRoots[37],	&gRoots[38],	&gRoots[39],
+	&gRoots[40],	&gRoots[41],	&gRoots[42],	&gRoots[43],
+	&gRoots[44],	&gRoots[45],	&gRoots[46],	&gRoots[47],
+	&gRoots[48],	&gRoots[49],	&gRoots[50],	&gRoots[51],
+	&gRoots[52],	&gRoots[53],	&gRoots[54],	&gRoots[55],
+	&gRoots[56],	&gRoots[57],	&gRoots[58],	&gRoots[59],
+	&gRoots[60],	&gRoots[61],	&gRoots[62],	&gRoots[63],
+};
+
+
 int				CProfileManager::FrameCounter = 0;
 unsigned long int			CProfileManager::ResetTime = 0;
 
+CProfileIterator *	CProfileManager::Get_Iterator( void )
+{ 
+
+		int threadIndex = btQuickprofGetCurrentThreadIndex2();
+		if (threadIndex<0)
+			return 0;
+
+		return new CProfileIterator( &gRoots[threadIndex]); 
+}
+
+void						CProfileManager::CleanupMemory(void)
+{
+	for (int i=0;i<BT_QUICKPROF_MAX_THREAD_COUNT;i++)
+	{
+		gRoots[i].CleanupMemory();
+	}
+}
+
 
 /***********************************************************************************************
  * CProfileManager::Start_Profile -- Begin a named profile                                    *
@@ -455,11 +548,15 @@ unsigned long int			CProfileManager::ResetTime = 0;
  *=============================================================================================*/
 void	CProfileManager::Start_Profile( const char * name )
 {
-	if (name != CurrentNode->Get_Name()) {
-		CurrentNode = CurrentNode->Get_Sub_Node( name );
+	int threadIndex = btQuickprofGetCurrentThreadIndex2();
+	if (threadIndex<0)
+		return;
+
+	if (name != gCurrentNodes[threadIndex]->Get_Name()) {
+		gCurrentNodes[threadIndex] = gCurrentNodes[threadIndex]->Get_Sub_Node( name );
 	}
 
-	CurrentNode->Call();
+	gCurrentNodes[threadIndex]->Call();
 }
 
 
@@ -468,14 +565,22 @@ void	CProfileManager::Start_Profile( const char * name )
  *=============================================================================================*/
 void	CProfileManager::Stop_Profile( void )
 {
+	int threadIndex = btQuickprofGetCurrentThreadIndex2();
+	if (threadIndex<0)
+		return;
+
 	// Return will indicate whether we should back up to our parent (we may
 	// be profiling a recursive function)
-	if (CurrentNode->Return()) {
-		CurrentNode = CurrentNode->Get_Parent();
+	if (gCurrentNodes[threadIndex]->Return()) {
+		gCurrentNodes[threadIndex] = gCurrentNodes[threadIndex]->Get_Parent();
 	}
 }
 
 
+
+
+
+
 /***********************************************************************************************
  * CProfileManager::Reset -- Reset the contents of the profiling system                       *
  *                                                                                             *
@@ -484,8 +589,11 @@ void	CProfileManager::Stop_Profile( void )
 void	CProfileManager::Reset( void )
 {
 	gProfileClock.reset();
-	Root.Reset();
-    Root.Call();
+	int threadIndex = btQuickprofGetCurrentThreadIndex2();
+	if (threadIndex<0)
+		return;
+	gRoots[threadIndex].Reset();
+	gRoots[threadIndex].Call();
 	FrameCounter = 0;
 	Profile_Get_Ticks(&ResetTime);
 }
@@ -575,4 +683,99 @@ void	CProfileManager::dumpAll()
 
 
 
+unsigned int btQuickprofGetCurrentThreadIndex2()
+{
+	const unsigned int kNullIndex = ~0U;
+#ifdef _WIN32
+	__declspec( thread ) static unsigned int sThreadIndex = kNullIndex;
+#else
+#ifdef __APPLE__
+	#if TARGET_OS_IPHONE
+		unsigned int sThreadIndex = 0;
+		return -1;
+	#else
+		static __thread unsigned int sThreadIndex = kNullIndex;
+	#endif
+#else//__APPLE__
+#if __linux__
+	static __thread unsigned int sThreadIndex = kNullIndex;
+#else
+	unsigned int sThreadIndex = 0;
+	return -1;
+#endif
+#endif//__APPLE__
+	
+#endif
+	static int gThreadCounter=0;
+
+	if ( sThreadIndex == kNullIndex )
+	{
+		sThreadIndex = gThreadCounter++;
+	}
+	return sThreadIndex;
+}
+
+void	btEnterProfileZoneDefault(const char* name)
+{
+	CProfileManager::Start_Profile( name ); 
+}
+void	btLeaveProfileZoneDefault()
+{
+	CProfileManager::Stop_Profile(); 
+}
+
+
+#else
+void	btEnterProfileZoneDefault(const char* name)
+{
+}
+void	btLeaveProfileZoneDefault()
+{
+}
 #endif //BT_NO_PROFILE
+
+
+
+
+
+static btEnterProfileZoneFunc* bts_enterFunc = btEnterProfileZoneDefault;
+static btLeaveProfileZoneFunc* bts_leaveFunc = btLeaveProfileZoneDefault;
+
+void btEnterProfileZone(const char* name)
+{
+	(bts_enterFunc)(name);
+}
+void btLeaveProfileZone()
+{
+	(bts_leaveFunc)();
+}
+
+btEnterProfileZoneFunc* btGetCurrentEnterProfileZoneFunc()
+{
+	return bts_enterFunc ;
+}
+btLeaveProfileZoneFunc* btGetCurrentLeaveProfileZoneFunc()
+{
+	return bts_leaveFunc;
+}
+
+
+void btSetCustomEnterProfileZoneFunc(btEnterProfileZoneFunc* enterFunc)
+{
+	bts_enterFunc = enterFunc;
+}
+void btSetCustomLeaveProfileZoneFunc(btLeaveProfileZoneFunc* leaveFunc)
+{
+	bts_leaveFunc = leaveFunc;
+}
+
+CProfileSample::CProfileSample( const char * name )
+{ 
+	btEnterProfileZone(name);
+}
+
+CProfileSample::~CProfileSample( void )					
+{ 
+	btLeaveProfileZone();
+}
+
